@@ -2,7 +2,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
-#include <orb/orbit.h>
 #include "empty.h"
 
 Empty empty_client = CORBA_OBJECT_NIL;
@@ -32,6 +31,7 @@ main (int argc, char *argv[])
     CORBA_Environment ev;
     char *retval;
     CORBA_ORB orb;
+    PortableServer_ObjectId *oid;
 
     signal(SIGINT, do_exit);
     signal(SIGTERM, do_exit);
@@ -43,16 +43,18 @@ main (int argc, char *argv[])
 
     poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
     PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(poa, &ev), &ev);
-    PortableServer_POA_activate_object_with_id(poa,
-					       &objid, &poa_empty_servant, &ev);
+    oid = PortableServer_POA_activate_object(poa, &poa_empty_servant, &ev);
+    if(ev._major == CORBA_NO_EXCEPTION)
+      CORBA_free(oid);
 
     empty_client = PortableServer_POA_servant_to_reference(poa,
-							&poa_empty_servant,
-							  &ev);
-    if (!empty_client) {
+							   &poa_empty_servant,
+							   &ev);
+    if (ev._major != CORBA_NO_EXCEPTION)
+      {
 	printf("Cannot get objref\n");
 	return 1;
-    }
+      }
 
     retval = CORBA_ORB_object_to_string(orb, empty_client, &ev);
 
@@ -62,11 +64,13 @@ main (int argc, char *argv[])
 
     CORBA_ORB_run(orb, &ev);
 
+    PortableServer_POA_deactivate_object(poa, &objid, &ev);
+
     return 0;
 }
 
 static void
 do_Nothing(PortableServer_Servant servant,
-	      CORBA_Environment *ev)
+	   CORBA_Environment *ev)
 {
 }
